package com.mortals.xhx.module.certificate.service.impl;
import cn.hutool.core.text.StrSplitter;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.template.MetaTemplate;
import com.deepoove.poi.util.RegexUtils;
import com.mortals.xhx.common.code.ComponentEnum;
import com.mortals.xhx.common.code.StatusEnum;
import com.mortals.xhx.common.formdesign.*;
import com.mortals.xhx.common.pdu.gen.component.ComponentCons;
import com.mortals.xhx.common.utils.ExcelUtil;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.certificate.model.CertificateCatalogEntity;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.certificate.dao.CertificateCatalogTemplateDao;
import com.mortals.xhx.module.certificate.model.CertificateCatalogTemplateEntity;
import com.mortals.xhx.module.certificate.service.CertificateCatalogTemplateService;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

/**
* CertificateCatalogTemplateService
* 证照目录模板 service实现
*
* @author zxfei
* @date 2023-05-23
*/
@Service("certificateCatalogTemplateService")
@Slf4j
public class CertificateCatalogTemplateServiceImpl extends AbstractCRUDServiceImpl<CertificateCatalogTemplateDao, CertificateCatalogTemplateEntity, Long> implements CertificateCatalogTemplateService {

    @Value("${upload.path}")
    private String filePath;

    @Value("${upload.url:http://192.168.0.98:17216/cpm/file/commonupload?prePath=/file/uploadfile}")
    private String uploadUrl;

    @Value("${qrcode.url:http://rhpt.scjg.chengdu.gov.cn:8091/adap/spjy/process/generateQrCode.do?dto.applyId=}")
    private String qrcode;

    @Override
    protected void saveBefore(CertificateCatalogTemplateEntity entity, Context context) throws AppException {

        if(entity.getCatalogId()==null){
            throw new AppException("证照目录不能为空");
        }
        entity.setStatus(StatusEnum.ENABLE.getValue());
        if(StringUtils.isNotEmpty(entity.getTemplateFileUrl())){
            //entity.setFormContent(parseDocxToJson(entity.getTemplateUrl()));
            parseDocxToJson(entity,false);
        }
    }

    @Override
    protected void updateBefore(CertificateCatalogTemplateEntity entity, Context context) throws AppException {

        CertificateCatalogTemplateEntity old = this.get(entity.getId());
        if(!old.getTemplateFileUrl().equals(entity.getTemplateFileUrl())) {
            if (StringUtils.isNotEmpty(entity.getTemplateFileUrl())) {
                //entity.setFormContent(parseDocxToJson(entity.getTemplateUrl()));
                parseDocxToJson(entity, false);
            }
        }

    }

    private void parseDocxToJson(CertificateCatalogTemplateEntity entity, boolean onlyExcel) {
        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
        String filepath = rootPath + entity.getTemplateFileUrl();
        ConfigureBuilder builder = Configure.builder();

        builder.setValidErrorHandler(new Configure.DiscardHandler());
        builder.useDefaultEL(false);
        builder.buildGrammerRegex(RegexUtils.createGeneral("{{", "}}"));
        //builder.buildGramer("[","]");
        XWPFTemplate template = XWPFTemplate.compile(filepath, builder.build());
        FormDesignRoot root = new FormDesignRoot();
        FormDesignConfig config = new FormDesignConfig();
        config.initAttribute();
        root.setConfig(config);
        List<ListItem> collect = new ArrayList<>();
        HashSet<String> hs = new HashSet<>(); //去除重复标签
        List<String> excelList = new ArrayList<>();      //Excel模板
        for (MetaTemplate item : template.getElementTemplates()) {
            if(hs.contains(item.variable())){
                continue;
            }else {
                hs.add(item.variable());
            }
            //识别简单输入
            String val = StrUtil.strip(item.variable(), "{{", "}}");
            excelList.add(val);
            List<String> keys = StrSplitter.split(val, "_", true, true);
            //切分分隔符后动态 创建组件  i_1_姓名（第一个代表类型，第二个如果是多个累加，第三个显示label）
            if (keys.size() > 2) {
                String label = keys.get(2);
                String type = keys.get(0);
                ComponentCons cons = new ComponentCons();
                cons.setLabel(label);
                cons.setVal(val);
                cons.setEl(item.variable());
                cons.setUploadFileUrl(uploadUrl);

                if (type.equalsIgnoreCase(ComponentEnum.DYNAMIC_TABLE.getValue())) {
                    HashSet<String> dhs = new HashSet<>(); //去除重复标签
                    List<TableColItem> colList = new ArrayList<>();
                    //表格 结构化绑定数据
                    ConfigureBuilder builderTable = Configure.builder();
                    builderTable.setValidErrorHandler(new Configure.DiscardHandler());
                    builderTable.useDefaultEL(false);
                    builderTable.buildGramer("[", "]");
                    XWPFTemplate templateTable = XWPFTemplate.compile(filepath, builderTable.build());

                    for (MetaTemplate col : templateTable.getElementTemplates()) {
                        if(dhs.contains(col.variable())){
                            continue;
                        }else {
                            dhs.add(col.variable());
                        }
                        String colVal = StrUtil.strip(col.variable(), "[", "]");
                        if (!StrUtil.startWith(colVal, label)) {
                            continue;
                        }
                        List<String> colKeys = StrSplitter.split(colVal, "_", true, true);
                        //提取当前table中的列元素
                        if (colKeys.size() > 3) {
                            String colLabel = colKeys.get(3);
                            String colType = colKeys.get(1);
                            TableColItem tableColItem = new TableColItem();
                            tableColItem.setLabel(colLabel);
                            tableColItem.setColType(colType);
                            tableColItem.setVal(colVal);
                            colList.add(tableColItem);
                            //System.out.println("列元素" + col.variable());
                        }
                    }
                    cons.setColList(colList);
                    DesignComponent designComponent = DesignComponent.createType(type);
                    ListItem listItem = designComponent.buildDefaultComponent(cons);
                    collect.add(listItem);
                    continue;
                }
                DesignComponent designComponent = DesignComponent.createType(type);
                ListItem listItem = designComponent.buildDefaultComponent(cons);
                if(type.equals(ComponentEnum.QRCODE.getValue())){
                    listItem.setValue(qrcode);
                }
                collect.add(listItem);
            }
        }
        root.setList(collect);
        String fileName = "file/uploadfile/excel/"+entity.getTemplateName()+"_批量导入模板.xls";
        String excelFilePath = rootPath + fileName;
        ExcelUtil.createCatalogTemplate(excelFilePath, excelList);
        entity.setExcelFile(fileName);
        if(!onlyExcel) {
            entity.setFormContent(JSON.toJSONString(root));
        }
    }
}